/* Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.model.sinfo;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.dive4elements.river.backend.SessionHolder;
import org.dive4elements.river.model.River;
import org.hibernate.Query;
import org.hibernate.Session;

/**
 * Hibernate binding for the DB table depth_evolution
 *
 * @author Matthias Schäfer
 *
 */

@Entity
@Table(name = "depth_evolution")
public class DepthEvolution implements Serializable {

    /***** TYPES *****/

    /**
     * Possible values of the groupname column
     *
     */
    public enum Group {
        NONE(""), AKTUELL("aktuell"), ETAPPE("etappe");

        private final String dbvalue;

        Group(final String dbvalue) {
            this.dbvalue = dbvalue;
        }

        /**
         * Value of the group's groupname column
         */
        public String getDbValue() {
            return this.dbvalue;
        }

        /**
         * Group for a groupname column value
         */
        public static Group forDbValue(final String dbvalue) {
            for (final Group g : Group.values()) {
                if (g.getDbValue().equals(dbvalue))
                    return g;
            }
            return Group.NONE;
        }
    }


    /***** FIELDS *****/

    private static final long serialVersionUID = 2665718161356165179L;

    private Integer id;

    private River river;

    private String kmrange_info;

    private String filename;

    private transient Group group;

    private String notes;

    private Integer start_year;

    private Integer reference_year;

    private String curr_sounding;

    private String old_sounding;

    private String curr_glw;

    private String old_glw;

    private List<DepthEvolutionValue> values;


    /***** CONSTRUCTORS *****/

    public DepthEvolution() {
    }

    public DepthEvolution(final River river, final String filename, final String kmrange_info, final String notes, final Integer start_year,
            final Integer reference_year, final String curr_sounding, final String old_sounding, final String curr_glw, final String old_glw,
            final Group group) {
        this.river = river;
        this.filename = filename;
        this.kmrange_info = kmrange_info;
        this.notes = notes;
        this.setStart_year(start_year);
        this.setReference_year(reference_year);
        this.curr_sounding = curr_sounding;
        this.old_sounding = old_sounding;
        this.curr_glw = curr_glw;
        this.old_glw = old_glw;
        this.group = group;
        this.values = new ArrayList<>();
    }

    /***** METHODS *****/

    @Id
    @SequenceGenerator(name = "SEQUENCE_DEPTH_EVOLUTION_ID_SEQ", sequenceName = "DEPTH_EVOLUTION_ID_SEQ", allocationSize = 1)
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQUENCE_DEPTH_EVOLUTION_ID_SEQ")
    @Column(name = "id")
    public Integer getId() {
        return this.id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    @OneToOne
    @JoinColumn(name = "river_id")
    public River getRiver() {
        return this.river;
    }

    public void setRiver(final River river) {
        this.river = river;
    }

    @Column(name = "filename")
    public String getFilename() {
        return this.filename;
    }

    public void setFilename(final String filename) {
        this.filename = filename;
    }

    @Column(name = "groupname")
    public String getGroupname() {
        return this.group.getDbValue();
    }

    public void setGroupname(final String groupname) {
        this.group = Group.forDbValue(groupname);
    }

    @Transient
    public Group getGroup() {
        return this.group;
    }

    public void setGroup(final Group group) {
        this.group = group;
    }

    @Column(name = "kmrange_info")
    public String getKmrange_info() {
        return this.kmrange_info;
    }

    public void setKmrange_info(final String kmrange_info) {
        this.kmrange_info = kmrange_info;
    }

    @Column(name = "notes")
    public String getNotes() {
        return this.notes;
    }

    public void setNotes(final String notes) {
        this.notes = notes;
    }

    @Column(name = "start_year")
    public Integer getStart_year() {
        return this.start_year;
    }

    public void setStart_year(final Integer start_year) {
        this.start_year = start_year;
    }

    @Column(name = "reference_year")
    public Integer getReference_year() {
        return this.reference_year;
    }

    public void setReference_year(final Integer reference_year) {
        this.reference_year = reference_year;
    }

    @Column(name = "curr_sounding")
    public String getCurr_sounding() {
        return this.curr_sounding;
    }

    public void setCurr_sounding(final String curr_sounding) {
        this.curr_sounding = curr_sounding;
    }

    @Column(name = "old_sounding")
    public String getOld_sounding() {
        return this.old_sounding;
    }

    public void setOld_sounding(final String old_sounding) {
        this.old_sounding = old_sounding;
    }

    @Column(name = "curr_glw")
    public String getCurr_glw() {
        return this.curr_glw;
    }

    public void setCurr_glw(final String curr_glw) {
        this.curr_glw = curr_glw;
    }

    @Column(name = "old_glw")
    public String getOld_glw() {
        return this.old_glw;
    }

    public void setOld_glw(final String old_glw) {
        this.old_glw = old_glw;
    }

    @OneToMany
    @JoinColumn(name = "depth_evolution_id")
    public List<DepthEvolutionValue> getValues() {
        return this.values;
    }

    public void setValues(final List<DepthEvolutionValue> values) {
        this.values = values;
    }

    public void addValue(final DepthEvolutionValue value) {
        this.values.add(value);
    }

    /**
     * Get data series by id
     */
    public static DepthEvolution getSeries(final int id) {
        final Session session = SessionHolder.HOLDER.get();
        final Query query = session.createQuery("FROM DepthEvolution WHERE (id=:id)");
        query.setParameter("id", id);
        return (DepthEvolution) query.list().get(0);
    }
}